@@ -1,17 +1,13 @@
-# -*- coding=utf-8 -*-
-from __future__ import absolute_import
-
 import os
 
-import click.types
-
-from click import (
-    BadParameter, BadArgumentUsage, Group, Option, argument, echo, make_pass_decorator, option
+from pipenv.project import Project
+from pipenv.utils import is_valid_url
+from pipenv.vendor.click import (
+    BadArgumentUsage, BadParameter, Group, Option, argument, echo,
+    make_pass_decorator, option
 )
-from click_didyoumean import DYMMixin
-
-from .. import environments
-from ..utils import is_valid_url
+from pipenv.vendor.click import types as click_types
+from pipenv.vendor.click_didyoumean import DYMMixin
 
 
 CONTEXT_SETTINGS = {
@@ -50,8 +46,14 @@ class PipenvGroup(DYMMixin, Group):
             help="Show this message and exit.",
         )
 
+    def main(self, *args, **kwargs):
+        """
+        to specify the windows_expand_args option to avoid exceptions on Windows
+        see: https://github.com/pallets/click/issues/1901
+        """
+        return super().main(*args, **kwargs, windows_expand_args=False)
 
-class State(object):
+class State:
     def __init__(self):
         self.index = None
         self.extra_index_urls = []
@@ -64,11 +66,12 @@ class State(object):
         self.site_packages = None
         self.clear = False
         self.system = False
+        self.project = Project()
         self.installstate = InstallState()
         self.lockoptions = LockOptions()
 
 
-class InstallState(object):
+class InstallState:
     def __init__(self):
         self.dev = False
         self.pre = False
@@ -84,7 +87,7 @@ class InstallState(object):
         self.editables = []
 
 
-class LockOptions(object):
+class LockOptions:
     def __init__(self):
         self.dev_only = False
         self.emit_requirements = False
@@ -110,7 +113,7 @@ def extra_index_option(f):
         state.extra_index_urls.extend(list(value))
         return value
     return option("--extra-index-url", multiple=True, expose_value=False,
-                  help=u"URLs to the extra PyPI compatible indexes to query for package look-ups.",
+                  help="URLs to the extra PyPI compatible indexes to query for package look-ups.",
                   callback=callback, envvar="PIP_EXTRA_INDEX_URL")(f)
 
 
@@ -120,7 +123,7 @@ def editable_option(f):
         state.installstate.editables.extend(value)
         return value
     return option('-e', '--editable', expose_value=False, multiple=True,
-                  callback=callback, type=click.types.STRING, help=(
+                  callback=callback, type=click_types.STRING, help=(
                       "An editable Python package URL or path, often to a VCS "
                       "repository."
                   ))(f)
@@ -133,7 +136,7 @@ def sequential_option(f):
         return value
     return option("--sequential", is_flag=True, default=False, expose_value=False,
                   help="Install dependencies one-at-a-time, instead of concurrently.",
-                  callback=callback, type=click.types.BOOL, show_envvar=True)(f)
+                  callback=callback, type=click_types.BOOL, show_envvar=True)(f)
 
 
 def skip_lock_option(f):
@@ -142,8 +145,8 @@ def skip_lock_option(f):
         state.installstate.skip_lock = value
         return value
     return option("--skip-lock", is_flag=True, default=False, expose_value=False,
-                  help=u"Skip locking mechanisms and use the Pipfile instead during operation.",
-                  envvar="PIPENV_SKIP_LOCK", callback=callback, type=click.types.BOOL,
+                  help="Skip locking mechanisms and use the Pipfile instead during operation.",
+                  envvar="PIPENV_SKIP_LOCK", callback=callback, type=click_types.BOOL,
                   show_envvar=True)(f)
 
 
@@ -153,8 +156,8 @@ def keep_outdated_option(f):
         state.installstate.keep_outdated = value
         return value
     return option("--keep-outdated", is_flag=True, default=False, expose_value=False,
-                  help=u"Keep out-dated dependencies from being updated in Pipfile.lock.",
-                  callback=callback, type=click.types.BOOL, show_envvar=True)(f)
+                  help="Keep out-dated dependencies from being updated in Pipfile.lock.",
+                  callback=callback, type=click_types.BOOL, show_envvar=True)(f)
 
 
 def selective_upgrade_option(f):
@@ -162,7 +165,7 @@ def selective_upgrade_option(f):
         state = ctx.ensure_object(State)
         state.installstate.selective_upgrade = value
         return value
-    return option("--selective-upgrade", is_flag=True, default=False, type=click.types.BOOL,
+    return option("--selective-upgrade", is_flag=True, default=False, type=click_types.BOOL,
                   help="Update specified packages.", callback=callback,
                   expose_value=False)(f)
 
@@ -174,7 +177,7 @@ def ignore_pipfile_option(f):
         return value
     return option("--ignore-pipfile", is_flag=True, default=False, expose_value=False,
                   help="Ignore Pipfile when installing, using the Pipfile.lock.",
-                  callback=callback, type=click.types.BOOL, show_envvar=True)(f)
+                  callback=callback, type=click_types.BOOL, show_envvar=True)(f)
 
 
 def _dev_option(f, help_text):
@@ -182,7 +185,7 @@ def _dev_option(f, help_text):
         state = ctx.ensure_object(State)
         state.installstate.dev = value
         return value
-    return option("--dev", "-d", is_flag=True, default=False, type=click.types.BOOL,
+    return option("--dev", "-d", is_flag=True, default=False, type=click_types.BOOL,
                   help=help_text, callback=callback,
                   expose_value=False, show_envvar=True)(f)
 
@@ -204,8 +207,8 @@ def pre_option(f):
         state = ctx.ensure_object(State)
         state.installstate.pre = value
         return value
-    return option("--pre", is_flag=True, default=False, help=u"Allow pre-releases.",
-                  callback=callback, type=click.types.BOOL, expose_value=False)(f)
+    return option("--pre", is_flag=True, default=False, help="Allow pre-releases.",
+                  callback=callback, type=click_types.BOOL, expose_value=False)(f)
 
 
 def package_arg(f):
@@ -214,7 +217,7 @@ def package_arg(f):
         state.installstate.packages.extend(value)
         return value
     return argument('packages', nargs=-1, callback=callback, expose_value=False,
-                    type=click.types.STRING)(f)
+                    type=click_types.STRING)(f)
 
 
 def three_option(f):
@@ -222,9 +225,8 @@ def three_option(f):
         state = ctx.ensure_object(State)
         if value is not None:
             state.three = value
-            state.two = not value
         return value
-    return option("--three/--two", is_flag=True, default=None,
+    return option("--three", is_flag=True, default=None,
                   help="Use Python 3/2 when creating virtualenv.", callback=callback,
                   expose_value=False)(f)
 
@@ -235,19 +237,21 @@ def python_option(f):
         if value is not None:
             state.python = validate_python_path(ctx, param, value)
         return value
-    return option("--python", default=False, nargs=1, callback=callback,
+    return option("--python", default="", nargs=1, callback=callback,
                   help="Specify which version of Python virtualenv should use.",
-                  expose_value=False, allow_from_autoenv=False)(f)
+                  expose_value=False, allow_from_autoenv=False,
+                  type=click_types.STRING)(f)
 
 
 def pypi_mirror_option(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
+        value = value or state.project.s.PIPENV_PYPI_MIRROR
         if value is not None:
             state.pypi_mirror = validate_pypi_mirror(ctx, param, value)
         return value
-    return option("--pypi-mirror", default=environments.PIPENV_PYPI_MIRROR, nargs=1,
-                  callback=callback, help="Specify a PyPI mirror.", expose_value=False)(f)
+    return option("--pypi-mirror", nargs=1, callback=callback,
+                  help="Specify a PyPI mirror.", expose_value=False)(f)
 
 
 def verbose_option(f):
@@ -262,7 +266,7 @@ def verbose_option(f):
             state.verbose = True
             setup_verbosity(ctx, param, 1)
     return option("--verbose", "-v", is_flag=True, expose_value=False,
-                  callback=callback, help="Verbose mode.", type=click.types.BOOL)(f)
+                  callback=callback, help="Verbose mode.", type=click_types.BOOL)(f)
 
 
 def quiet_option(f):
@@ -277,7 +281,7 @@ def quiet_option(f):
             state.quiet = True
             setup_verbosity(ctx, param, -1)
     return option("--quiet", "-q", is_flag=True, expose_value=False,
-                  callback=callback, help="Quiet mode.", type=click.types.BOOL)(f)
+                  callback=callback, help="Quiet mode.", type=click_types.BOOL)(f)
 
 
 def site_packages_option(f):
@@ -296,8 +300,8 @@ def clear_option(f):
         state = ctx.ensure_object(State)
         state.clear = value
         return value
-    return option("--clear", is_flag=True, callback=callback, type=click.types.BOOL,
-                  help="Clears caches (pipenv, pip, and pip-tools).",
+    return option("--clear", is_flag=True, callback=callback, type=click_types.BOOL,
+                  help="Clears caches (pipenv, pip).",
                   expose_value=False, show_envvar=True)(f)
 
 
@@ -308,7 +312,7 @@ def system_option(f):
             state.system = value
         return value
     return option("--system", is_flag=True, default=False, help="System pip management.",
-                  callback=callback, type=click.types.BOOL, expose_value=False,
+                  callback=callback, type=click_types.BOOL, expose_value=False,
                   show_envvar=True)(f)
 
 
@@ -318,8 +322,9 @@ def requirementstxt_option(f):
         if value:
             state.installstate.requirementstxt = value
         return value
-    return option("--requirements", "-r", nargs=1, default=False, expose_value=False,
-                  help="Import a requirements.txt file.", callback=callback)(f)
+    return option("--requirements", "-r", nargs=1, default="", expose_value=False,
+                  help="Import a requirements.txt file.", callback=callback,
+                  type=click_types.STRING)(f)
 
 
 def emit_requirements_flag(f):
@@ -352,24 +357,13 @@ def dev_only_flag(f):
                   help="Emit development dependencies *only* (overrides --dev)", callback=callback)(f)
 
 
-def code_option(f):
-    def callback(ctx, param, value):
-        state = ctx.ensure_object(State)
-        if value:
-            state.installstate.code = value
-        return value
-    return option("--code", "-c", nargs=1, default=False, help="Install packages "
-                  "automatically discovered from import statements.", callback=callback,
-                  expose_value=False)(f)
-
-
 def deploy_option(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
         state.installstate.deploy = value
         return value
-    return option("--deploy", is_flag=True, default=False, type=click.types.BOOL,
-                  help=u"Abort if the Pipfile.lock is out-of-date, or Python version is"
+    return option("--deploy", is_flag=True, default=False, type=click_types.BOOL,
+                  help="Abort if the Pipfile.lock is out-of-date, or Python version is"
                   " wrong.", callback=callback, expose_value=False)(f)
 
 
@@ -377,14 +371,14 @@ def setup_verbosity(ctx, param, value):
     if not value:
         return
     import logging
-    loggers = ("pip", "piptools")
+    loggers = ("pip",)
     if value == 1:
         for logger in loggers:
             logging.getLogger(logger).setLevel(logging.INFO)
     elif value == -1:
         for logger in loggers:
             logging.getLogger(logger).setLevel(logging.CRITICAL)
-    environments.PIPENV_VERBOSITY = value
+    ctx.ensure_object(State).project.s.PIPENV_VERBOSITY = value
 
 
 def validate_python_path(ctx, param, value):
@@ -401,7 +395,7 @@ def validate_python_path(ctx, param, value):
 
 def validate_bool_or_none(ctx, param, value):
     if value is not None:
-        return click.types.BOOL(value)
+        return click_types.BOOL(value)
     return False
 
 
@@ -414,6 +408,7 @@ def validate_pypi_mirror(ctx, param, value):
 def common_options(f):
     f = pypi_mirror_option(f)
     f = verbose_option(f)
+    f = quiet_option(f)
     f = clear_option(f)
     f = three_option(f)
     f = python_option(f)
